/**
 * Created by Caos 2016-09-13
 */

import RNFetchBlob from 'react-native-fetch-blob';
import defaultConfig from './config';
var md5 = require('md5');
import FileMD5 from 'react-native-filemd5';


// var md5File = require('md5-file');

export default class Files {

    constructor(config = defaultConfig) {

        this._config = config;
    }

    /**
     * 下载文件方法
     * @param {string} url 文件地址
     *
     */
    getRes(url) {
        return RNFetchBlob.fetch('GET', url, {})
        // when response status code is 200
            .then((res) => {
                // the conversion is done in native code
                let base64Str = res.base64()
                // the following conversions are done in js, it's SYNC
            })
    }

    /**
     * 上传文件方法
     * @param {string} filePath 远程地址
     * @param {object} option 参数
     *          @property {string} appid .
     *          @property {object} plus 可选,post时携带的数据,headers及data参见react-native-fetch-blob文档
     *          @property {function} 进度数据的回调函数 function(written,total){ ... }
     *          @property {boolean} 是否开启秒传功能,默认为 false
     * @param {boolean} isImage
     * @returns {Promise} 返回值可调用cancel方法取消任务.
     */
    upload(filePath, options, progressFn, flash = false, isImage) {
        if (!filePath) {
            return null;
        }
        let params = {
            appid: options.appid,
            random: parseInt(Math.random() * 1000000),
            timestamp: (new Date().getTime() / 1000).toPrecision(10)
        }
        let sign = this.getSignedString(params, this._config.secretKey);
        let uploadPath = this._config.uploadPath;

        let md5Str = ''
        if (flash) {
            return FileMD5(filePath).then((data) => {
                md5Str = data
                console.log(md5Str)
                let filename = filePath.split("/").pop();
                let type = filePath.split(".").pop();
                return RNFetchBlob.fetch('POST', uploadPath, {
                    Authorization: "Bearer access-token",
                    "Content-Type": 'multipart/form-data',
                    'File-MD5': md5Str,
                }, [
                    {name: 'file', filename: filename, type: type, data: RNFetchBlob.wrap(filePath)},
                    // elements without property `filename` will be sent as plain text
                    {name: 'random', data: params.random},
                    {name: 'appid', data: params.appid},
                    {name: 'timestamp', data: params.timestamp},
                    {name: 'sign', data: sign},
                    {name: 'isImage', data: isImage},
                    {name: 'plus', data: JSON.stringify(options.plus || {})},
                ]).uploadProgress((written, total) => {
                    if (progressFn) {
                        progressFn(written, total);
                    }
                })
            });
        } else {
            let filename = filePath.split("/").pop();
            let type = filePath.split(".").pop();
            return RNFetchBlob.fetch('POST', this._config.uploadPath, {
                Authorization: "Bearer access-token",
                "Content-Type": 'multipart/form-data',
            }, [
                {name: 'file', filename: filename, type: type, data: RNFetchBlob.wrap(filePath)},
                // elements without property `filename` will be sent as plain text
                {name: 'random', data: params.random},
                {name: 'appid', data: params.appid},
                {name: 'timestamp', data: params.timestamp},
                {name: 'sign', data: sign},
                {name: 'plus', data: JSON.stringify(options.plus || {})},
            ]).uploadProgress((written, total) => {
                if (progressFn) {
                    progressFn(written, total);
                }
            })
        }


    }

    getSignedString(obj, key) {
        var oldStr = '';
        var sortedList = this.sortParams(obj);
        for (var i = 0; i < sortedList.length; i++) {
            oldStr += sortedList[i].key + '=' + sortedList[i].value + '&';
        }
        console.log(oldStr);
        oldStr += 'key=' + key;
        console.log('clinet:',oldStr);
        var sign = md5(oldStr);
        return sign;
    }

    sortParams(obj) {
        var mapList = [];
        for (var item in obj) {
            if (item != "sign" && item != "plus") {
                var o = {};
                o.key = item;
                o.value = obj[item];
                mapList.push(o);
            }
        }
        mapList.sort(function (a, b) {
            return a.key > b.key;
        });
        return mapList;
    }

    /*fileMD5(path) {
        let data = ''
        return new Promise(function (resolve, reject) {
            return RNFetchBlob.fs.readStream(path, 'ascii').then((stream)=> {
                let data = '';
                var md5sum = crypto.createHash('md5');
                stream.open();
                stream.onData((chunk) => {
                    // data += chunk
                    chunk = Uint8Array.from(chunk)
                    md5sum.update(chunk)
                })

                stream.onError((err) => {
                    console.log('oops', err)
                })
                stream.onEnd(() => {
                   let str = md5sum.digest('hex');
                    resolve(str)
                })
            })
        })
    }*/

}

// let FileOpt = new Files()
